import { JSDOM } from 'jsdom';
import axios from 'axios';
import puppeteer from 'puppeteer';
import get from 'lodash.get';
import xpath, { XPathResult } from 'xpath-ts';

var EngineType;
(function (EngineType) {
    EngineType["Html"] = "html";
    EngineType["Spa"] = "spa";
    EngineType["Json"] = "json";
    EngineType["Xml"] = "xml";
})(EngineType || (EngineType = {}));
var PluginType;
(function (PluginType) {
    PluginType["Transformer"] = "transformer";
    PluginType["Engine"] = "engine";
})(PluginType || (PluginType = {}));

const DEFAULT_HTTP_REQUESTS_OPTIONS = {
    headers: {
        "Cache-Control": "no-cache",
        "Accept-Encoding": "gzip, deflate",
    },
};

const request$1 = async (url, request = {}) => {
    try {
        const response = await axios.get(url, {
            ...DEFAULT_HTTP_REQUESTS_OPTIONS,
            ...request,
        });
        return response.data;
    }
    catch (error) {
        throw new Error(`Error fetching data from ${url}: ${error.message}`);
    }
};

const getSelectorValue$2 = (element, selector) => {
    const value = selector.attribute
        ? element.getAttribute(selector.attribute) || ""
        : element.textContent?.trim() || "";
    return value;
};
const extractData$2 = (document, selectors) => {
    const data = {};
    for (const [key, selector] of Object.entries(selectors)) {
        const elements = document.querySelectorAll(selector.selector);
        if (!elements.length) {
            data[key] = "";
        }
        else if (elements.length === 1) {
            data[key] = getSelectorValue$2(elements[0], selector) || "";
        }
        else {
            data[key] = Array.from(elements)
                .map((element) => getSelectorValue$2(element, selector))
                .filter((value) => value !== null);
        }
    }
    return data;
};

const processData$3 = async (url, options) => {
    const { selectors, request, plugins } = options;
    let html = await request$1(url, request);
    plugins?.forEach((plugin) => {
        if (plugin.preProcess) {
            html = plugin.preProcess(html);
        }
    });
    const dom = new JSDOM(html);
    const document = dom.window.document;
    return extractData$2(document, selectors);
};
const scrapeHtml = async (url, { selectors, request, plugins }) => {
    plugins?.forEach((plugin) => {
        if (plugin.pluginType === "transformer" &&
            plugin.initialize &&
            plugin.supportedEngines?.includes("html")) {
            plugin.initialize({ selectors });
        }
    });
    let data = await processData$3(url, { selectors, request, plugins });
    plugins?.forEach((plugin) => {
        if (plugin.postProcess) {
            data = plugin.postProcess(data);
        }
    });
    return data;
};

const autoScroll = async (page) => {
    await page.evaluate(async () => {
        await new Promise((resolve) => {
            let totalHeight = 0;
            let distance = 100;
            let timer = setInterval(() => {
                let scrollHeight = document.body.scrollHeight;
                window.scrollBy(0, distance);
                totalHeight += distance;
                if (totalHeight >= scrollHeight) {
                    clearInterval(timer);
                    resolve(true);
                }
            }, 100);
        });
    });
};
const request = async (url, config = {}) => {
    const browser = await puppeteer.launch({
        executablePath: config?.executablePath
            ? config.executablePath
            : puppeteer.executablePath(),
        ...config,
    });
    const page = await browser.newPage();
    await page.goto(url);
    await autoScroll(page);
    const content = await page.content();
    await browser.close();
    return content;
};

const processData$2 = async (url, options) => {
    const { selectors, request: request$1, plugins } = options;
    let html = await request(url, request$1);
    plugins?.forEach((plugin) => {
        if (plugin.preProcess) {
            html = plugin.preProcess(html);
        }
    });
    const dom = new JSDOM(html);
    const document = dom.window.document;
    return extractData$2(document, selectors);
};
const scrapeSpa = async (url, { selectors, request, plugins }) => {
    plugins?.forEach((plugin) => {
        if (plugin.pluginType === "transformer" &&
            plugin.initialize &&
            plugin?.supportedEngines?.includes("spa")) {
            plugin.initialize({ selectors });
        }
    });
    let data = await processData$2(url, { selectors, request, plugins });
    plugins?.forEach((plugin) => {
        if (plugin.postProcess) {
            data = plugin.postProcess(data);
        }
    });
    return data;
};

const getSelectorValue$1 = (value, selector) => {
    if (typeof value !== "string") {
        value = JSON.stringify(value);
    }
    return value;
};
const extractData$1 = (j, selectors) => {
    const data = {};
    if (Array.isArray(j)) {
        return j
            .map((obj) => {
            const data = {};
            for (const [key, selector] of Object.entries(selectors)) {
                const value = get(obj, selector.selector, "");
                data[key] = getSelectorValue$1(value);
            }
            return data;
        })
            .filter((item) => Object.values(item).some((value) => value !== null));
    }
    for (const [key, selector] of Object.entries(selectors)) {
        const value = get(j, selector.selector, "");
        data[key] = getSelectorValue$1(value);
    }
    return data;
};

const processData$1 = async (url, { selectors, request, plugins }) => {
    let j = await request$1(url, request);
    plugins?.forEach((plugin) => {
        if (plugin.preProcess) {
            j = plugin.preProcess(j);
        }
    });
    return extractData$1(j, selectors);
};
const scrapeJson = async (url, { selectors, request, plugins }) => {
    plugins?.forEach((plugin) => {
        if (plugin.pluginType === "transformer" &&
            plugin.initialize &&
            plugin?.supportedEngines?.includes("json")) {
            plugin.initialize({ selectors });
        }
    });
    let data = await processData$1(url, { selectors, request, plugins });
    plugins?.forEach((plugin) => {
        if (plugin.postProcess) {
            data = plugin.postProcess(data);
        }
    });
    return data;
};

const getSelectorValue = (values, selector) => {
    return values.length === 1 ? values[0] : values;
};
const extractData = (document, selectors) => {
    const data = {};
    for (const [key, value] of Object.entries(selectors)) {
        const nodes = xpath.evaluate(value.selector, document, null, XPathResult.ANY_TYPE, null);
        const nodeValues = [];
        let node = nodes.iterateNext();
        while (node) {
            nodeValues.push(node.textContent || "");
            node = nodes.iterateNext();
        }
        data[key] = getSelectorValue(nodeValues);
    }
    return data;
};

const processData = async (url, { selectors, request, plugins }) => {
    let xml = await request$1(url, request);
    plugins?.forEach((plugin) => {
        if (plugin.preProcess) {
            xml = plugin.preProcess(xml);
        }
    });
    const dom = new JSDOM(xml, { contentType: "text/xml" });
    const document = dom.window.document;
    return extractData(document, selectors);
};
const scrapeXml = async (url, { selectors, request, plugins }) => {
    plugins?.forEach((plugin) => {
        if (plugin.pluginType === "transformer" &&
            plugin.initialize &&
            plugin?.supportedEngines?.includes("xml")) {
            plugin.initialize({ selectors });
        }
    });
    let data = await processData(url, { selectors, request, plugins });
    plugins?.forEach((plugin) => {
        if (plugin.postProcess) {
            data = plugin.postProcess(data);
        }
    });
    return data;
};

const SCRAPE_TYPES = {
    html: scrapeHtml,
    spa: scrapeSpa,
    json: scrapeJson,
    xml: scrapeXml,
};
const scrape = async ({ url, engine, options, }) => {
    return await SCRAPE_TYPES[engine](url, options);
};

const operators = {
    difference: (a, b, sensitive) => sensitive ? a !== b : a.toLowerCase() !== b.toLowerCase(),
    equal: (a, b, sensitive) => sensitive ? a === b : a.toLowerCase() === b.toLowerCase(),
    contains: (a, b, sensitive) => sensitive ? a.includes(b) : a.toLowerCase().includes(b.toLowerCase()),
    regex: (a, b, sensitive) => new RegExp(b, sensitive ? "" : "i").test(a),
};
const applyConditions = (value, conditions) => {
    return conditions.every((condition) => {
        const operation = operators[condition.operation];
        return operation
            ? operation(value, condition.value, condition.sensitive || false)
            : false;
    });
};

const plugin = (conditions) => ({
    supportedEngines: ["html", "json", "xml", "spa"],
    pluginType: "transformer",
    postProcess(data) {
        const newData = {};
        for (const [key, value] of Object.entries(data)) {
            if (conditions[key]) {
                if (Array.isArray(value)) {
                    newData[key] = value.filter((v) => applyConditions(v, conditions[key]));
                }
                else {
                    newData[key] = applyConditions(value, conditions[key]) ? value : null;
                }
            }
            else {
                newData[key] = value;
            }
        }
        return newData;
    },
});

var ConditionOperationType;
(function (ConditionOperationType) {
    ConditionOperationType["Difference"] = "difference";
    ConditionOperationType["Equal"] = "equal";
    ConditionOperationType["Contains"] = "contains";
    ConditionOperationType["Regex"] = "regex";
})(ConditionOperationType || (ConditionOperationType = {}));

export { ConditionOperationType, EngineType, PluginType, plugin as conditionsPlugin, scrape, scrapeHtml, scrapeJson, scrapeSpa, scrapeXml };
//# sourceMappingURL=index.js.map
